app = {
	desktop = application:getDeviceInfo() == "Windows"
		or application:getDeviceInfo() == "Mac OS",
	fontName = "fonts/BlissProExtraLight.otf",
	lang = application:getLanguage(),
	translation = {},
	T = function(s) return app.translation[s] or s end,
	fontSize = math.ceil(65536 / application:getDeviceHeight()),
	fontCache = "",

}

function string:split(d)
	local t = {}
	for line in self:gmatch("[^"..d.."]*") do
		if line~="" then t[#t+1] = line end
	end
	return t
end

function string:trim()
	local n1 = 0
	for i = 1, #self do
		if self:byte(i) <= 32 then n1 = n1 + 1 else break end
	end
	
	local n2 = 0
	for i = #self, 1, -1 do
		if self:byte(i) <= 32 then n2 = n2 + 1 else break end			
	end

	return self:sub(1 + n1, #self - n2)
end

local file = io.open("|R|langs/"..app.lang..".ini")
if file then
	local data = file:read("*a")
	file:close()
	local pos = 1
	for i = 1, #data do if data:byte(i) < 128 then pos = i; break end end
	if pos > 1 then data = data:sub(pos) end
	for k,v in ipairs(data:split "\n") do
		local t = v:split "="
		local key, value = t[1]:trim(), t[2] and t[2]:trim()
		app.translation[key] = value
	end
else
	print("lang file '"..app.lang..".ini' is not found")
end


function toUTF8(decimal)
if decimal < 128 then return string.char(decimal) end
local charbytes = {}
for bytes,vals in ipairs { {0x7FF,192}, {0xFFFF,224}, {0x1FFFFF,240} } do
	if decimal <= vals[1] then
		for b = bytes+1, 2, -1 do
			local mod = decimal % 64
			decimal = (decimal - mod) / 64
			charbytes[b] = string.char(128 + mod)
		end
		charbytes[1] = string.char(vals[2] + decimal)
		break
	end
end
return table.concat(charbytes)
end

UnicodeCharSets = {
	{0x0020,0x007F},
	{0x0080,0x009F},
	{0x00A0,0x00FF},
	{0x0100,0x017F},
	{0x0180,0x024F},
	{0x0250,0x02AF},
	{0x02B0,0x02FF},
	{0x0300,0x036F},
	{0x0370,0x03FF},
	{0x0400,0x04FF},
	{0x0500,0x052F},
	{0x0530,0x058F},
}

for k,v in ipairs(UnicodeCharSets) do
	for i = v[1], v[2] do app.fontCache = app.fontCache .. toUTF8(i) end
end

function app.loadFont(fontName, fontHeight, cellHeight)
	local font = TTFont.new(fontName, fontHeight, app.fontCache, false)
	font.offset = math.floor(0.25 * font:getAscender() - 0.5 * cellHeight)
	return font
end

app.font = TTFont.new(app.fontName, app.fontSize, app.fontCache, false)

require "json"
require "lfs"
require "utf8"

lfs.mkdir "|D|fields"

function warn(...)
	local text = ""
	for k,v in ipairs{...} do
		local t = type(v)
		if t == "table" then
			local isOk, s = xpcall(json.encode, v)
			if isOk then text = text .. s
			else
				for key,val in pairs(v) do
					if type(val) ~= "table" then
						text = text .. tostring(key) .. ": " .. tostring(val) .. "; "
					end
				end
			end
		else text = text .. tostring(v) end
		text = text .. " "
	end
	print('<pre><font size = 3 color = "#FF8888">'..text..'</font></pre>')
end

function fitText(s, maxWidth, font)
	local t = {}
	local min, max = 1, utf8.len(s)
	local pos = max
	while true do
		local s1 = utf8.sub(s, 1, pos)
		local w = font:getAdvanceX(s1)
		if w < maxWidth then min = pos else max = pos end
		if max - min < 2 then
			if max ~= pos or max == min then
				t[#t+1] = s1
				s = utf8.sub(s,pos+1)
				min, max = 1, utf8.len(s)
				if max < 1 then break end
			else
				pos = pos - 1
			end
		else pos = math.floor( (min + max) / 2 ) end
	end
	return t
end